"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = require("crypto");
const fs_1 = require("fs");
const apollo_engine_reporting_1 = require("apollo-engine-reporting");
const Command_1 = require("../../Command");
const manifestOperationHash = (str) => crypto_1.createHash("sha256")
    .update(str)
    .digest("hex");
const engineSignature = (_TODO_operationAST) => {
    return apollo_engine_reporting_1.defaultSignature(_TODO_operationAST, "TODO");
};
class ServicePush extends Command_1.ClientCommand {
    async run() {
        const { clientIdentity, operations, filename } = await this.runTasks(({ flags, project, config, args }) => [
            {
                title: "Extracting operations from project",
                task: async (ctx) => {
                    const operations = Object.values(this.project.mergedOperationsAndFragmentsForService).map(operationAST => {
                        const printed = apollo_engine_reporting_1.printWithReducedWhitespace(apollo_engine_reporting_1.sortAST(apollo_engine_reporting_1.hideLiterals(operationAST)));
                        return {
                            signature: manifestOperationHash(printed),
                            document: printed,
                            metadata: {
                                engineSignature: engineSignature(operationAST)
                            }
                        };
                    });
                    ctx.operations = operations;
                    ctx.clientIdentity = config.client;
                }
            },
            {
                title: "Outputing extracted queries",
                task: (ctx, task) => {
                    const filename = args.output;
                    task.title = "Outputing extracted queries to " + filename;
                    ctx.filename = filename;
                    fs_1.writeFileSync(filename, JSON.stringify({ version: 1, operations: ctx.operations }, null, 2));
                }
            }
        ]);
        this.log(`Successfully wrote ${operations.length} operations from the ${clientIdentity.name} client to ${filename}`);
    }
}
ServicePush.description = "Push a service to Engine";
ServicePush.flags = Object.assign({}, Command_1.ClientCommand.flags);
ServicePush.args = [
    {
        name: "output",
        description: "Path to write the extracted queries to",
        required: true,
        default: "manifest.json"
    }
];
exports.default = ServicePush;
//# sourceMappingURL=extract.js.map