"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const heroku_cli_util_1 = require("heroku-cli-util");
const graphql_1 = require("graphql");
const git_1 = require("../../git");
const Command_1 = require("../../Command");
class ServicePush extends Command_1.ProjectCommand {
    async run() {
        let result;
        let gitContext;
        await this.runTasks(({ flags, project, config }) => [
            {
                title: "Uploading service to Engine",
                task: async () => {
                    if (!config.name) {
                        throw new Error("No service name found to link to Engine");
                    }
                    const schema = await project.resolveSchema({ tag: flags.tag });
                    gitContext = await git_1.gitInfo();
                    const { tag, code } = await project.engine.uploadSchema({
                        id: config.name,
                        schema: graphql_1.introspectionFromSchema(schema).__schema,
                        tag: flags.tag,
                        gitContext
                    });
                    result = {
                        service: config.name,
                        hash: tag.schema.hash,
                        tag: tag.tag,
                        code
                    };
                }
            }
        ]);
        this.log("\n");
        if (result.code === "NO_CHANGES") {
            this.log("No change in schema from previous version\n");
        }
        heroku_cli_util_1.table([result], {
            columns: [
                {
                    key: "hash",
                    label: "id",
                    format: (hash) => hash.slice(0, 6)
                },
                { key: "service", label: "schema" },
                { key: "tag" }
            ]
        });
        this.log("\n");
    }
}
ServicePush.aliases = ["schema:publish"];
ServicePush.description = "Push a service to Engine";
ServicePush.flags = Object.assign({}, Command_1.ProjectCommand.flags, { tag: command_1.flags.string({
        char: "t",
        description: "The published tag to check this service against",
        default: "current"
    }) });
exports.default = ServicePush;
//# sourceMappingURL=push.js.map